import logging

import cx_Oracle

from .abstract import Abstract_DB
from soft_collect.config import settings as s

logger = logging.getLogger(__name__)


class Oracle(Abstract_DB):
    def select(self, sql):
        c = self.set_up_connection()

        yield from c.execute(sql)

    def fetch_all(self, sql):
        c = self.set_up_connection()

        return c.execute(sql).fetchall()

    def set_up_connection(self):
        logger.info(f"Creating connection to SQL Server in {s.ip}")
        connection = cx_Oracle.connect(
            s.user, s.password, f"{s.ip}/{s.base}", encoding="UTF-8", nencoding="UTF-8"
        )

        return connection.cursor()



import asyncio
import functools
import io
import logging
from concurrent.futures import ThreadPoolExecutor

import pandas as pd

from .config import settings
from .db import get_DB_client
from .save import get_save_client
from .utils import progress_bar

logger = logging.getLogger(__name__)


class Collect:
    def __init__(self, save):
        self.sh = get_save_client(save)
        self.db = get_DB_client(settings.dbm)

    async def run(self, classes):
        loop = asyncio.get_event_loop()
        executor = ThreadPoolExecutor(10)
        with progress_bar as p:
            class_task = p.add_task("[red]CLASSES", total=len(classes))
            for cdclass in classes:
                doc_task = p.add_task(f"[green]DOCUMENTS FOR CLASS {cdclass}")
                logger.info(f"Getting documents for class {cdclass}")
                await asyncio.gather(
                    *[
                        loop.run_in_executor(
                            executor,
                            functools.partial(
                                self.parallel_docs, p, doc_task, doc, cdclass
                            ),
                        )
                        for doc in self.get_keys(cdclass)
                    ]
                )
                p.stop_task(doc_task)
                logger.info(f"Finished class {cdclass}")
                p.advance(class_task)

    def parallel_docs(self, progress, task, doc, cdclass):
        logger.info(f"Getting key {doc} in class {cdclass}")
        self.get_objects(doc, cdclass)
        progress.advance(task)

    def get_objects(self, key, cdclass):
        if self.sh.check_obj(cdclass, key):
            logger.info(f"Object {key} already exists")
            return True
        for key, part, obj in self.db.get_objects(key):
            if isinstance(obj, bytes):
                obj = io.BytesIO(obj)
            self.sh.save_obj(obj.read(), cdclass, key, part)

    def get_keys(self, cdclass):
        exists, docs = self.sh.check_meta(f"DOCS_LIST_{cdclass}")
        if exists:
            logger.info("List of keys for class {cdclass} already exists")
            yield from [doc[0] for _, doc in docs.iterrows()]
        else:
            docs = self.db.get_keys(cdclass)
            docs = [doc[0] for doc in docs]
            self.sh.save_meta(pd.DataFrame(docs), f"DOCS_LIST_{cdclass}")
            yield from docs



import logging

import pandas as pd

from .config import settings
from .db import get_DB_client
from .utils import elapsed_time
from .utils import print_table

logger = logging.getLogger(__name__)


class Query:
    def __init__(self):
        self.db = get_DB_client(settings.dbm)

    def run(self, sql, filename):
        logger.info(f"Reading SQL file in {sql}")
        with open(sql, "r") as f:
            query = f.read()
        logger.info("Running query")
        with elapsed_time as et:
            task = et.add_task("Running your query, elapsed time:")
            result = self.db.fetch_all(query)
            et.stop_task(task)
        logger.info("Finished running query")
        df = pd.DataFrame(result)
        print_table(df.head(50), "Your query first 50 results")
        logger.info(f"Saving result to {filename}")
        df.to_csv(filename, index=False, header=False)



import logging

import pyodbc

from .abstract import Abstract_DB
from soft_collect.config import settings

logger = logging.getLogger(__name__)


class SQLServer(Abstract_DB):
    def select(self, sql):
        c = self.set_up_connection()
        c.execute(sql)
        row = c.fetchone()
        while row:
            yield row
            row = c.fetchone()

    def fetch_all(self, sql):
        c = self.set_up_connection()

        return c.execute(sql).fetchall()

    def set_up_connection(self):
        s = settings
        logger.info(f"Creating connection to SQL Server in {s.ip}")

        driver = "ODBC Driver 17 for SQL Server"
        pyodbc_drivers = [dri for dri in pyodbc.drivers() if "SQL Server" in dri]
        if pyodbc_drivers:
            driver = pyodbc_drivers[0]
        if s.get("DRIVER", None):
            driver = s.driver
        driver_str = "Driver={" + driver + "};"

        connection = pyodbc.connect(
            f"{driver_str}"
            f"Server={s.ip};"
            f"Database={s.base};"
            f"UID={s.user};"
            f"PWD={s.password};"
        )
        return connection.cursor()
